from __future__ import annotations

from openai import OpenAI
from openai.types.beta import BetaResponseOutputItem

PROMPT = """Compare these fictional proposals on speed, cost, reliability, and security.
Proposal Alpha: 2-week launch, $40k, managed vendor, 99.9% SLA, data leaves the VPC.
Proposal Beta: 6-week launch, $70k, self-hosted, 99.5% target, data stays in the VPC.
Delegate each proposal to a separate agent, then compare their evidence and recommend one."""


def agent_name(item: BetaResponseOutputItem) -> str:
    return item.agent.agent_name if item.agent else "/root"


client = OpenAI()

stream = client.beta.responses.create(
    model="gpt-5.6-sol",
    input=PROMPT,
    multi_agent={"enabled": True},
    stream=True,
    betas=["responses_multi_agent=v1"],
)

item_agents: dict[int, str] = {}
labeled_items: set[int] = set()
for event in stream:
    if event.type == "response.output_item.added":
        item_agents[event.output_index] = agent_name(event.item)
    elif event.type == "response.output_text.delta":
        name = item_agents.get(event.output_index, "/root")
        if event.output_index not in labeled_items:
            separator = "\n\n" if labeled_items else ""
            role = "Coordinator" if name == "/root" else "Agent"
            print(f"{separator}━━━ {role}: {name} ━━━\n")
            labeled_items.add(event.output_index)
        print(event.delta, end="", flush=True)

print()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ProjectDataRetention"]


class ProjectDataRetention(BaseModel):
    """Represents a project's data retention control setting."""

    object: Literal["project.data_retention"]
    """The object type, which is always `project.data_retention`."""

    type: Literal[
        "organization_default",
        "none",
        "zero_data_retention",
        "modified_abuse_monitoring",
        "enhanced_zero_data_retention",
        "enhanced_modified_abuse_monitoring",
    ]
    """The configured project data retention type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ProjectUser"]


class ProjectUser(BaseModel):
    """Represents an individual user in a project."""

    id: str
    """The identifier, which can be referenced in API endpoints"""

    added_at: int
    """The Unix timestamp (in seconds) of when the project was added."""

    object: Literal["organization.project.user"]
    """The object type, which is always `organization.project.user`"""

    role: str
    """`owner` or `member`"""

    email: Optional[str] = None
    """The email address of the user"""

    name: Optional[str] = None
    """The name of the user"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .beta_computer_action import BetaComputerAction
from .beta_computer_action_list import BetaComputerActionList

__all__ = ["BetaResponseComputerToolCall", "PendingSafetyCheck", "Agent"]


class PendingSafetyCheck(BaseModel):
    """A pending safety check for the computer call."""

    id: str
    """The ID of the pending safety check."""

    code: Optional[str] = None
    """The type of the pending safety check."""

    message: Optional[str] = None
    """Details about the pending safety check."""


class Agent(BaseModel):
    """The agent that produced this item."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class BetaResponseComputerToolCall(BaseModel):
    """A tool call to a computer use tool.

    See the
    [computer use guide](https://platform.openai.com/docs/guides/tools-computer-use) for more information.
    """

    id: str
    """The unique ID of the computer call."""

    call_id: str
    """An identifier used when responding to the tool call with output."""

    pending_safety_checks: List[PendingSafetyCheck]
    """The pending safety checks for the computer call."""

    status: Literal["in_progress", "completed", "incomplete"]
    """The status of the item.

    One of `in_progress`, `completed`, or `incomplete`. Populated when items are
    returned via API.
    """

    type: Literal["computer_call"]
    """The type of the computer call. Always `computer_call`."""

    action: Optional[BetaComputerAction] = None
    """A click action."""

    actions: Optional[BetaComputerActionList] = None
    """Flattened batched actions for `computer_use`.

    Each action includes an `type` discriminator and action-specific fields.
    """

    agent: Optional[Agent] = None
    """The agent that produced this item."""



from __future__ import annotations

import base64
from typing import Any
from typing_extensions import override

import httpx
import pytest

import openai
from openai import (
    OpenAI,
    AsyncOpenAI,
    AzureOpenAI,
    OpenAIError,
    APIStatusError,
    APITimeoutError,
    AsyncAzureOpenAI,
    APIConnectionError,
)
from openai._response import StreamAlreadyConsumed
from openai.providers import bedrock
from openai._constants import DEFAULT_TIMEOUT

httpx2 = pytest.importorskip("httpx2")


@pytest.fixture(autouse=True)
def forbid_httpx_execution(monkeypatch: pytest.MonkeyPatch) -> None:
    def forbidden(*_args: object, **_kwargs: object) -> None:
        raise AssertionError("the experimental path unexpectedly executed an HTTPX client")

    monkeypatch.setattr(httpx.Client, "build_request", forbidden)
    monkeypatch.setattr(httpx.Client, "send", forbidden)
    monkeypatch.setattr(httpx.AsyncClient, "build_request", forbidden)
    monkeypatch.setattr(httpx.AsyncClient, "send", forbidden)


def model_list(request: httpx.Request) -> httpx.Response:
    return httpx2.Response(200, json={"object": "list", "data": []}, request=request)


def sse_response(request: httpx.Request) -> httpx.Response:
    return httpx2.Response(
        200,
        headers={"content-type": "text/event-stream"},
        content=(
            b'data: {"type":"response.completed","response":{"id":"resp_test","object":"response",'
            b'"created_at":0,"model":"gpt-4o","output":[],"parallel_tool_calls":false,'
            b'"tool_choice":"auto","tools":[]}}\n\ndata: [DONE]\n\n'
        ),
        request=request,
    )


async def test_httpx2_helpers_supply_sdk_defaults_and_accept_native_proxy() -> None:
    sync_client = openai.DefaultHttpx2Client(proxy="http://127.0.0.1:8080", trust_env=False)
    async_client = openai.DefaultAsyncHttpx2Client(proxy="http://127.0.0.1:8080", trust_env=False)
    try:
        assert type(sync_client).__module__ == "httpx2"
        assert type(async_client).__module__ == "httpx2"
        assert sync_client.timeout.as_dict() == {"connect": 5.0, "read": 600, "write": 600, "pool": 600}
        assert async_client.timeout.as_dict() == {"connect": 5.0, "read": 600, "write": 600, "pool": 600}
        assert sync_client.follow_redirects
        assert async_client.follow_redirects
    finally:
        sync_client.close()
        await async_client.aclose()


def test_sync_helper_preserves_httpx2_family_for_parsed_raw_and_sse() -> None:
    requests: list[httpx.Request] = []
    hooks: list[str] = []

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return sse_response(request) if request.url.path.endswith("/responses") else model_list(request)

    def on_request(request: httpx.Request) -> None:
        hooks.append(type(request).__module__)

    with OpenAI(
        api_key="test",
        base_url=httpx2.URL("https://example.test/v1"),
        http_client=openai.DefaultHttpx2Client(
            timeout=httpx.Timeout(30.0, read=10.0),
            auth=httpx2.BasicAuth("fake-test-user", "fake-test-password"),
            headers=[("x-repeated", "one"), ("x-repeated", "two")],
            mounts={"https://example.test": httpx2.MockTransport(handler)},
            event_hooks={"request": [on_request]},
            trust_env=False,
        ),
        max_retries=0,
    ) as client:
        parsed = client.models.list(extra_query={"tag": ["one", "two"]})
        raw = client.models.with_raw_response.list()
        stream = client.responses.create(model="gpt-4o", input="hello", stream=True)
        events = list(stream)
        multipart = client.post(
            "/multipart",
            files={"file": ("example.txt", b"body", "text/plain")},
            options={"headers": {"Content-Type": "multipart/form-data"}},
            cast_to=httpx.Response,
        )

    assert parsed.object == "list"
    assert type(raw.http_response).__module__ == "httpx2"
    assert type(raw.http_request).__module__ == "httpx2"
    assert type(stream.response).__module__ == "httpx2"
    assert [event.type for event in events] == ["response.completed"]
    assert type(multipart).__module__ == "httpx2"
    assert hooks == ["httpx2", "httpx2", "httpx2", "httpx2"]
    assert all(type(request).__module__ == "httpx2" for request in requests)
    assert (
        requests[0].headers["authorization"]
        == f"Basic {base64.b64encode(b'fake-test-user:fake-test-password').decode()}"
    )
    assert requests[0].headers.get_list("x-repeated") == ["one", "two"]
    assert requests[0].url.params.get_list("tag[]") == ["one", "two"]
    assert requests[0].extensions["timeout"] == {"connect": 30.0, "read": 10.0, "write": 30.0, "pool": 30.0}
    assert requests[-1].headers["content-type"].startswith("multipart/form-data; boundary=")


async def test_async_helper_preserves_httpx2_family_for_parsed_raw_and_sse() -> None:
    requests: list[httpx.Request] = []
    hooks: list[str] = []

    async def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return sse_response(request) if request.url.path.endswith("/responses") else model_list(request)

    async def on_request(request: httpx.Request) -> None:
        hooks.append(type(request).__module__)

    async with AsyncOpenAI(
        api_key="test",
        base_url=httpx2.URL("https://example.test/v1"),
        http_client=openai.DefaultAsyncHttpx2Client(
            timeout=httpx.Timeout(30.0, read=10.0),
            auth=httpx2.BasicAuth("fake-test-user", "fake-test-password"),
            headers=[("x-repeated", "one"), ("x-repeated", "two")],
            transport=httpx2.MockTransport(handler),
            event_hooks={"request": [on_request]},
            trust_env=False,
        ),
        max_retries=0,
    ) as client:
        parsed = await client.models.list(extra_query={"tag": ["one", "two"]})
        raw = await client.models.with_raw_response.list()
        stream = await client.responses.create(model="gpt-4o", input="hello", stream=True)
        events = [event async for event in stream]
        multipart = await client.post(
            "/multipart",
            files={"file": ("example.txt", b"body", "text/plain")},
            options={"headers": {"Content-Type": "multipart/form-data"}},
            cast_to=httpx.Response,
        )

    assert parsed.object == "list"
    assert type(raw.http_response).__module__ == "httpx2"
    assert type(raw.http_request).__module__ == "httpx2"
    assert type(stream.response).__module__ == "httpx2"
    assert [event.type for event in events] == ["response.completed"]
    assert type(multipart).__module__ == "httpx2"
    assert hooks == ["httpx2", "httpx2", "httpx2", "httpx2"]
    assert all(type(request).__module__ == "httpx2" for request in requests)
    assert (
        requests[0].headers["authorization"]
        == f"Basic {base64.b64encode(b'fake-test-user:fake-test-password').decode()}"
    )
    assert requests[0].headers.get_list("x-repeated") == ["one", "two"]
    assert requests[0].url.params.get_list("tag[]") == ["one", "two"]
    assert requests[0].extensions["timeout"] == {"connect": 30.0, "read": 10.0, "write": 30.0, "pool": 30.0}
    assert requests[-1].headers["content-type"].startswith("multipart/form-data; boundary=")


def test_direct_sync_injection_and_module_configuration() -> None:
    direct = httpx2.Client(transport=httpx2.MockTransport(model_list), trust_env=False)
    with OpenAI(api_key="test", base_url="https://example.test/v1", http_client=direct, max_retries=0) as client:
        assert client.timeout == DEFAULT_TIMEOUT
        assert client.models.list().object == "list"

    openai.api_key = "test"
    openai.base_url = httpx2.URL("https://example.test/v1")
    openai.http_client = openai.DefaultHttpx2Client(transport=httpx2.MockTransport(model_list), trust_env=False)
    try:
        response = openai.models.with_raw_response.list()
    finally:
        openai._reset_client()
        openai.http_client = None

    assert type(response.http_response).__module__ == "httpx2"


async def test_httpx2_urls_and_response_casts() -> None:
    class ResponseSubclass(httpx2.Response):
        pass

    def model(request: httpx.Request) -> httpx.Response:
        return httpx2.Response(
            200,
            request=request,
            json={"id": "gpt-4o", "object": "model", "created": 1, "owned_by": "openai"},
        )

    with OpenAI(
        api_key="test",
        base_url=httpx2.URL("https://example.test/v1"),
        http_client=httpx2.Client(transport=httpx2.MockTransport(model), trust_env=False),
        max_retries=0,
    ) as client:
        client.base_url = httpx2.URL("https://example.test/v1")
        response = client.get("/models", cast_to=httpx2.Response)
        legacy_raw = client.models.with_raw_response.retrieve("gpt-4o")
        with client.models.with_streaming_response.retrieve("gpt-4o") as streaming_raw:
            streaming_response = streaming_raw.parse(to=httpx2.Response)

        with pytest.raises(ValueError, match="Subclasses of HTTP response classes"):
            client.get("/models", cast_to=ResponseSubclass)

    async def handler(request: httpx.Request) -> httpx.Response:
        return model(request)

    async with AsyncOpenAI(
        api_key="test",
        base_url=httpx2.URL("https://example.test/v1"),
        http_client=httpx2.AsyncClient(transport=httpx2.MockTransport(handler), trust_env=False),
        max_retries=0,
    ) as async_client:
        async_client.base_url = httpx2.URL("https://example.test/v1")
        async_response = await async_client.get("/models", cast_to=httpx2.Response)
        async_legacy_raw = await async_client.models.with_raw_response.retrieve("gpt-4o")
        async with async_client.models.with_streaming_response.retrieve("gpt-4o") as async_streaming_raw:
            async_streaming_response = await async_streaming_raw.parse(to=httpx2.Response)

        with pytest.raises(ValueError, match="Subclasses of HTTP response classes"):
            await async_client.get("/models", cast_to=ResponseSubclass)

    assert isinstance(response, httpx2.Response)
    assert isinstance(legacy_raw.parse(to=httpx2.Response), httpx2.Response)
    assert isinstance(streaming_response, httpx2.Response)
    assert isinstance(async_response, httpx2.Response)
    assert isinstance(async_legacy_raw.parse(to=httpx2.Response), httpx2.Response)
    assert isinstance(async_streaming_response, httpx2.Response)


async def test_httpx2_urls_work_for_bedrock_and_azure_realtime() -> None:
    with OpenAI(
        provider=bedrock(
            region="us-east-1",
            api_key="bedrock-token",
            base_url=httpx2.URL("https://bedrock.test/openai/v1/responses"),
        ),
        http_client=httpx2.Client(transport=httpx2.MockTransport(model_list), trust_env=False),
        max_retries=0,
    ) as client:
        assert client.models.list().object == "list"

    azure = AzureOpenAI(
        api_key="test",
        api_version="2024-02-01",
        azure_endpoint="https://azure.test",
        websocket_base_url=httpx2.URL("https://azure.test/openai/v1"),
        http_client=httpx2.Client(transport=httpx2.MockTransport(model_list), trust_env=False),
    )
    realtime_url, _ = azure._configure_realtime("gpt-4o", {})
    azure.close()

    async_azure = AsyncAzureOpenAI(
        api_key="test",
        api_version="2024-02-01",
        azure_endpoint="https://azure.test",
        websocket_base_url=httpx2.URL("https://azure.test/openai/v1"),
        http_client=httpx2.AsyncClient(transport=httpx2.MockTransport(model_list), trust_env=False),
    )
    async_realtime_url, _ = await async_azure._configure_realtime("gpt-4o", {})
    await async_azure.close()

    assert str(realtime_url).startswith("https://azure.test/openai/v1/realtime?")
    assert str(async_realtime_url).startswith("https://azure.test/openai/v1/realtime?")


async def test_httpx2_urls_work_for_all_websocket_builders() -> None:
    with OpenAI(
        api_key="test",
        websocket_base_url=httpx2.URL("wss://example.test/openai/v1"),
        http_client=httpx2.Client(transport=httpx2.MockTransport(model_list), trust_env=False),
    ) as client:
        assert str(client.realtime.connect()._prepare_url()) == "wss://example.test/openai/v1/realtime"
        assert str(client.responses.connect()._prepare_url()) == "wss://example.test/openai/v1/responses"
        assert (
            str(client.beta.realtime.connect(model="gpt-4o")._prepare_url()) == "wss://example.test/openai/v1/realtime"
        )
        assert str(client.beta.responses.connect()._prepare_url()) == "wss://example.test/openai/v1/responses"

    async with AsyncOpenAI(
        api_key="test",
        websocket_base_url=httpx2.URL("wss://example.test/openai/v1"),
        http_client=httpx2.AsyncClient(transport=httpx2.MockTransport(model_list), trust_env=False),
    ) as async_client:
        assert str(async_client.realtime.connect()._prepare_url()) == "wss://example.test/openai/v1/realtime"
        assert str(async_client.responses.connect()._prepare_url()) == "wss://example.test/openai/v1/responses"
        assert (
            str(async_client.beta.realtime.connect(model="gpt-4o")._prepare_url())
            == "wss://example.test/openai/v1/realtime"
        )
        assert str(async_client.beta.responses.connect()._prepare_url()) == "wss://example.test/openai/v1/responses"


async def test_httpx2_native_timeouts_set_numeric_read_timeout_header() -> None:
    sync_requests: list[httpx.Request] = []
    async_requests: list[httpx.Request] = []

    def sync_handler(request: httpx.Request) -> httpx.Response:
        sync_requests.append(request)
        return model_list(request)

    async def async_handler(request: httpx.Request) -> httpx.Response:
        async_requests.append(request)
        return model_list(request)

    with OpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        timeout=httpx2.Timeout(30.0, read=12.0),
        http_client=httpx2.Client(transport=httpx2.MockTransport(sync_handler), trust_env=False),
        max_retries=0,
    ) as client:
        client.models.list()
        client.models.list(timeout=httpx2.Timeout(30.0, read=7.0))

    async with AsyncOpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        timeout=httpx2.Timeout(30.0, read=13.0),
        http_client=httpx2.AsyncClient(transport=httpx2.MockTransport(async_handler), trust_env=False),
        max_retries=0,
    ) as async_client:
        await async_client.models.list()
        await async_client.models.list(timeout=httpx2.Timeout(30.0, read=8.0))

    assert [request.headers["x-stainless-read-timeout"] for request in sync_requests] == ["12.0", "7.0"]
    assert [request.headers["x-stainless-read-timeout"] for request in async_requests] == ["13.0", "8.0"]


async def test_direct_async_injection() -> None:
    async def handler(request: httpx.Request) -> httpx.Response:
        return model_list(request)

    direct = httpx2.AsyncClient(transport=httpx2.MockTransport(handler), trust_env=False)
    async with AsyncOpenAI(
        api_key="test", base_url="https://example.test/v1", http_client=direct, max_retries=0
    ) as client:
        assert client.timeout == DEFAULT_TIMEOUT
        assert (await client.models.list()).object == "list"


@pytest.mark.parametrize("failure", ["timeout", "connection", "status"])
def test_sync_retries_and_failure_families(failure: str, monkeypatch: pytest.MonkeyPatch) -> None:
    requests: list[httpx.Request] = []

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        if len(requests) > 1:
            return model_list(request)
        if failure == "timeout":
            raise httpx2.ReadTimeout("timeout", request=request)
        if failure == "connection":
            raise httpx2.ConnectError("connection", request=request)
        return httpx2.Response(500, json={"error": {"message": "bad", "type": "test"}}, request=request)

    client = OpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=httpx2.Client(transport=httpx2.MockTransport(handler), trust_env=False),
        max_retries=1,
    )

    def no_sleep(**_kwargs: Any) -> None:
        return None

    monkeypatch.setattr(client, "_sleep_for_retry", no_sleep)
    try:
        assert client.models.list().object == "list"
    finally:
        client.close()

    assert len(requests) == 2
    assert all(type(request).__module__ == "httpx2" for request in requests)

    def always_fail(request: httpx.Request) -> httpx.Response:
        if failure == "timeout":
            raise httpx2.ReadTimeout("timeout", request=request)
        if failure == "connection":
            raise httpx2.ConnectError("connection", request=request)
        return httpx2.Response(500, json={"error": {"message": "bad", "type": "test"}}, request=request)

    expected: type[Exception]
    if failure == "timeout":
        expected = APITimeoutError
    elif failure == "connection":
        expected = APIConnectionError
    else:
        expected = APIStatusError

    with OpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=httpx2.Client(transport=httpx2.MockTransport(always_fail), trust_env=False),
        max_retries=0,
    ) as failing_client:
        with pytest.raises(expected) as exc_info:
            failing_client.models.list()

    error = exc_info.value
    transport_value = getattr(error, "response", None) or getattr(error, "request", None)
    assert type(transport_value).__module__ == "httpx2"


@pytest.mark.parametrize("failure", ["timeout", "connection", "status"])
async def test_async_retries_and_failure_families(failure: str, monkeypatch: pytest.MonkeyPatch) -> None:
    requests: list[httpx.Request] = []

    async def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        if len(requests) > 1:
            return model_list(request)
        if failure == "timeout":
            raise httpx2.ReadTimeout("timeout", request=request)
        if failure == "connection":
            raise httpx2.ConnectError("connection", request=request)
        return httpx2.Response(500, json={"error": {"message": "bad", "type": "test"}}, request=request)

    client = AsyncOpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=httpx2.AsyncClient(transport=httpx2.MockTransport(handler), trust_env=False),
        max_retries=1,
    )

    async def no_sleep(**_kwargs: Any) -> None:
        return None

    monkeypatch.setattr(client, "_sleep_for_retry", no_sleep)
    try:
        assert (await client.models.list()).object == "list"
    finally:
        await client.close()

    assert len(requests) == 2
    assert all(type(request).__module__ == "httpx2" for request in requests)

    async def always_fail(request: httpx.Request) -> httpx.Response:
        if failure == "timeout":
            raise httpx2.ReadTimeout("timeout", request=request)
        if failure == "connection":
            raise httpx2.ConnectError("connection", request=request)
        return httpx2.Response(500, json={"error": {"message": "bad", "type": "test"}}, request=request)

    expected: type[Exception]
    if failure == "timeout":
        expected = APITimeoutError
    elif failure == "connection":
        expected = APIConnectionError
    else:
        expected = APIStatusError

    async with AsyncOpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=httpx2.AsyncClient(transport=httpx2.MockTransport(always_fail), trust_env=False),
        max_retries=0,
    ) as failing_client:
        with pytest.raises(expected) as exc_info:
            await failing_client.models.list()

    error = exc_info.value
    transport_value = getattr(error, "response", None) or getattr(error, "request", None)
    assert type(transport_value).__module__ == "httpx2"


async def test_provider_auth_and_stream_consumed_families() -> None:
    sync_requests: list[httpx.Request] = []
    async_requests: list[httpx.Request] = []

    def sync_handler(request: httpx.Request) -> httpx.Response:
        sync_requests.append(request)
        return model_list(request)

    async def async_handler(request: httpx.Request) -> httpx.Response:
        async_requests.append(request)
        return model_list(request)

    with OpenAI(
        provider=bedrock(region="us-east-1", api_key="bedrock-token", base_url="https://bedrock.test/openai/v1"),
        http_client=openai.DefaultHttpx2Client(transport=httpx2.MockTransport(sync_handler), trust_env=False),
        max_retries=0,
    ) as sync_client:
        assert sync_client.models.list().object == "list"

    async with AsyncOpenAI(
        provider=bedrock(region="us-east-1", api_key="bedrock-token", base_url="https://bedrock.test/openai/v1"),
        http_client=openai.DefaultAsyncHttpx2Client(transport=httpx2.MockTransport(async_handler), trust_env=False),
        max_retries=0,
    ) as async_client:
        assert (await async_client.models.list()).object == "list"

    assert sync_requests[0].headers["authorization"] == "Bearer bedrock-token"
    assert async_requests[0].headers["authorization"] == "Bearer bedrock-token"

    class SyncStream(httpx2.SyncByteStream):
        def __iter__(self):
            yield b'{"object":"list","data":[]}'

    class AsyncStream(httpx2.AsyncByteStream):
        async def __aiter__(self):
            yield b'{"object":"list","data":[]}'

    class FailingSyncStream(httpx2.SyncByteStream):
        def __iter__(self):
            yield b"partial"
            raise httpx2.ReadTimeout("stream timeout")

    class FailingAsyncStream(httpx2.AsyncByteStream):
        async def __aiter__(self):
            yield b"partial"
            raise httpx2.ReadTimeout("stream timeout")

    def sync_stream_handler(request: httpx.Request) -> httpx.Response:
        return httpx2.Response(200, headers={"content-type": "application/json"}, stream=SyncStream(), request=request)

    async def async_stream_handler(request: httpx.Request) -> httpx.Response:
        return httpx2.Response(200, headers={"content-type": "application/json"}, stream=AsyncStream(), request=request)

    def sync_failing_stream_handler(request: httpx.Request) -> httpx.Response:
        return httpx2.Response(
            200, headers={"content-type": "application/json"}, stream=FailingSyncStream(), request=request
        )

    async def async_failing_stream_handler(request: httpx.Request) -> httpx.Response:
        return httpx2.Response(
            200, headers={"content-type": "application/json"}, stream=FailingAsyncStream(), request=request
        )

    with OpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=httpx2.Client(transport=httpx2.MockTransport(sync_stream_handler), trust_env=False),
        max_retries=0,
    ) as sync_client:
        with sync_client.models.with_streaming_response.list() as response:
            assert b"".join(response.iter_bytes()) == b'{"object":"list","data":[]}'
            with pytest.raises(StreamAlreadyConsumed):
                response.read()

    with OpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=httpx2.Client(transport=httpx2.MockTransport(sync_failing_stream_handler), trust_env=False),
        max_retries=0,
    ) as sync_client:
        with sync_client.models.with_streaming_response.list() as response:
            with pytest.raises(httpx2.ReadTimeout, match="stream timeout"):
                list(response.iter_bytes())

    async with AsyncOpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=httpx2.AsyncClient(transport=httpx2.MockTransport(async_stream_handler), trust_env=False),
        max_retries=0,
    ) as async_client:
        async with async_client.models.with_streaming_response.list() as response:
            assert b"".join([chunk async for chunk in response.iter_bytes()]) == b'{"object":"list","data":[]}'
            with pytest.raises(StreamAlreadyConsumed):
                await response.read()

    async with AsyncOpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=httpx2.AsyncClient(transport=httpx2.MockTransport(async_failing_stream_handler), trust_env=False),
        max_retries=0,
    ) as async_client:
        async with async_client.models.with_streaming_response.list() as response:
            with pytest.raises(httpx2.ReadTimeout, match="stream timeout"):
                [chunk async for chunk in response.iter_bytes()]


@pytest.mark.filterwarnings("ignore:The Assistants API is deprecated in favor of the Responses API:DeprecationWarning")
async def test_assistant_stream_timeout_callbacks_preserve_httpx2_family() -> None:
    class SyncHandler(openai.AssistantEventHandler):
        def __init__(self) -> None:
            super().__init__()
            self.timed_out = False
            self.exception: Exception | None = None

        @override
        def on_timeout(self) -> None:
            self.timed_out = True

        @override
        def on_exception(self, exception: Exception) -> None:
            self.exception = exception

    class AsyncHandler(openai.AsyncAssistantEventHandler):
        def __init__(self) -> None:
            super().__init__()
            self.timed_out = False
            self.exception: Exception | None = None

        @override
        async def on_timeout(self) -> None:
            self.timed_out = True

        @override
        async def on_exception(self, exception: Exception) -> None:
            self.exception = exception

    class FailingSyncStream(httpx2.SyncByteStream):
        def __iter__(self):
            yield b"partial"
            raise httpx2.ReadTimeout("assistant stream timeout")

    class FailingAsyncStream(httpx2.AsyncByteStream):
        async def __aiter__(self):
            yield b"partial"
            raise httpx2.ReadTimeout("assistant stream timeout")

    def sync_response(request: httpx.Request) -> httpx.Response:
        return httpx2.Response(
            200, headers={"content-type": "text/event-stream"}, stream=FailingSyncStream(), request=request
        )

    async def async_response(request: httpx.Request) -> httpx.Response:
        return httpx2.Response(
            200, headers={"content-type": "text/event-stream"}, stream=FailingAsyncStream(), request=request
        )

    sync_handler = SyncHandler()
    with OpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=openai.DefaultHttpx2Client(transport=httpx2.MockTransport(sync_response), trust_env=False),
        max_retries=0,
    ) as sync_client:
        with sync_client.beta.threads.runs.stream(  # pyright: ignore[reportDeprecated]
            assistant_id="asst_test", thread_id="thread_test", event_handler=sync_handler
        ) as stream:
            with pytest.raises(httpx2.ReadTimeout, match="assistant stream timeout"):
                stream.until_done()

    assert sync_handler.timed_out
    assert isinstance(sync_handler.exception, httpx2.ReadTimeout)

    async_handler = AsyncHandler()
    async with AsyncOpenAI(
        api_key="test",
        base_url="https://example.test/v1",
        http_client=openai.DefaultAsyncHttpx2Client(transport=httpx2.MockTransport(async_response), trust_env=False),
        max_retries=0,
    ) as async_client:
        async with async_client.beta.threads.runs.stream(  # pyright: ignore[reportDeprecated]
            assistant_id="asst_test", thread_id="thread_test", event_handler=async_handler
        ) as async_stream:
            with pytest.raises(httpx2.ReadTimeout, match="assistant stream timeout"):
                await async_stream.until_done()

    assert async_handler.timed_out
    assert isinstance(async_handler.exception, httpx2.ReadTimeout)


async def test_sigv4_provider_preserves_httpx2_family_and_rejects_one_shot_bodies() -> None:
    pytest.importorskip("botocore")

    sync_requests: list[httpx.Request] = []
    async_requests: list[httpx.Request] = []

    def sync_handler(request: httpx.Request) -> httpx.Response:
        sync_requests.append(request)
        return model_list(request)

    async def async_handler(request: httpx.Request) -> httpx.Response:
        async_requests.append(request)
        return model_list(request)

    provider = bedrock(
        region="us-east-1",
        access_key_id="fixture-access-key",
        secret_access_key="fixture-secret-key",
        session_token="fixture-session-token",
        base_url="https://bedrock-mantle.us-east-1.api.aws/openai/v1",
    )

    with OpenAI(
        provider=provider,
        http_client=openai.DefaultHttpx2Client(transport=httpx2.MockTransport(sync_handler), trust_env=False),
        max_retries=0,
    ) as sync_client:
        sync_client.post("/responses", content=b"body", cast_to=httpx.Response)
        with pytest.raises(OpenAIError, match="requires a replayable request body"):
            sync_client.post("/responses", content=iter([b"body"]), cast_to=httpx.Response)

    async def body():
        yield b"body"

    async with AsyncOpenAI(
        provider=provider,
        http_client=openai.DefaultAsyncHttpx2Client(transport=httpx2.MockTransport(async_handler), trust_env=False),
        max_retries=0,
    ) as async_client:
        await async_client.post("/responses", content=b"body", cast_to=httpx.Response)
        with pytest.raises(OpenAIError, match="requires a replayable request body"):
            await async_client.post("/responses", content=body(), cast_to=httpx.Response)

    assert len(sync_requests) == 1
    assert len(async_requests) == 1
    assert "Credential=fixture-access-key/" in sync_requests[0].headers["authorization"]
    assert "Credential=fixture-access-key/" in async_requests[0].headers["authorization"]
    assert sync_requests[0].headers["x-amz-security-token"] == "fixture-session-token"
    assert async_requests[0].headers["x-amz-security-token"] == "fixture-session-token"



from __future__ import annotations

import os
from typing import Any, NoReturn

import httpx
import pytest
from respx import MockRouter

from openai import DefaultHttpx2Client, DefaultAsyncHttpx2Client


def httpx2_enabled() -> bool:
    return os.environ.get("OPENAI_TEST_HTTP_CLIENT") == "httpx2"


def sync_http_client(**kwargs: Any) -> httpx.Client:
    if httpx2_enabled():
        return DefaultHttpx2Client(**kwargs)
    return httpx.Client(**kwargs)


def async_http_client(**kwargs: Any) -> httpx.AsyncClient:
    if httpx2_enabled():
        return DefaultAsyncHttpx2Client(**kwargs)
    return httpx.AsyncClient(**kwargs)


def enable_httpx2_respx(
    router: MockRouter, monkeypatch: pytest.MonkeyPatch, *, replace_sdk_defaults: bool = True
) -> None:
    httpx2 = pytest.importorskip("httpx2")

    # RESPX deliberately only understands HTTPX objects. Keep the SDK side native and
    # translate at this single test-only boundary so existing routes, callbacks, and
    # call assertions can be exercised against HTTPX2 without copied test cases.
    def httpx_request(native_request: Any, *, content: bytes) -> httpx.Request:
        return httpx.Request(
            native_request.method,
            str(native_request.url),
            headers=list(native_request.headers.multi_items()),
            content=content,
            extensions=dict(native_request.extensions),
        )

    def httpx2_response(response: httpx.Response, *, native_request: Any, content: bytes) -> Any:
        return httpx2.Response(
            response.status_code,
            headers=list(response.headers.multi_items()),
            # Supplying a stream keeps streaming-response tests meaningful: the native
            # response stays open until the SDK consumes or closes it.
            stream=httpx2.ByteStream(content),
            request=native_request,
            extensions=dict(response.extensions),
        )

    def raise_native_request_error(exc: httpx.RequestError, native_request: Any) -> NoReturn:
        native_error = getattr(httpx2, type(exc).__name__, httpx2.RequestError)
        raise native_error(str(exc), request=native_request) from exc

    def handler(native_request: Any) -> Any:
        try:
            response = router.handler(httpx_request(native_request, content=native_request.read()))
        except httpx.RequestError as exc:
            raise_native_request_error(exc, native_request)
        return httpx2_response(response, native_request=native_request, content=response.read())

    async def async_handler(native_request: Any) -> Any:
        try:
            response = await router.async_handler(httpx_request(native_request, content=await native_request.aread()))
        except httpx.RequestError as exc:
            raise_native_request_error(exc, native_request)
        return httpx2_response(response, native_request=native_request, content=await response.aread())

    # Match RESPX's own patch point. This also catches clients created inside a test,
    # while leaving every non-RESPX test on its ordinary transport.
    def sync_transport(*_args: Any) -> Any:
        return httpx2.MockTransport(handler)

    def async_transport(*_args: Any) -> Any:
        return httpx2.MockTransport(async_handler)

    monkeypatch.setattr(httpx2.Client, "_transport_for_url", sync_transport)
    monkeypatch.setattr(httpx2.AsyncClient, "_transport_for_url", async_transport)

    if replace_sdk_defaults:
        import openai._base_client as base_client

        # Clients constructed inside a RESPX test should exercise the same native
        # path as the shared fixtures; the base-compatibility tests opt out.
        monkeypatch.setattr(base_client, "SyncHttpxClientWrapper", DefaultHttpx2Client)
        monkeypatch.setattr(base_client, "AsyncHttpxClientWrapper", DefaultAsyncHttpx2Client)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, TypeAlias

from .realtime_truncation_retention_ratio_param import RealtimeTruncationRetentionRatioParam

__all__ = ["RealtimeTruncationParam"]

RealtimeTruncationParam: TypeAlias = Union[Literal["auto", "disabled"], RealtimeTruncationRetentionRatioParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["InputAudioBufferTimeoutTriggered"]


class InputAudioBufferTimeoutTriggered(BaseModel):
    """Returned when the Server VAD timeout is triggered for the input audio buffer.

    This is configured
    with `idle_timeout_ms` in the `turn_detection` settings of the session, and it indicates that
    there hasn't been any speech detected for the configured duration.

    The `audio_start_ms` and `audio_end_ms` fields indicate the segment of audio after the last
    model response up to the triggering time, as an offset from the beginning of audio written
    to the input audio buffer. This means it demarcates the segment of audio that was silent and
    the difference between the start and end values will roughly match the configured timeout.

    The empty audio will be committed to the conversation as an `input_audio` item (there will be a
    `input_audio_buffer.committed` event) and a model response will be generated. There may be speech
    that didn't trigger VAD but is still detected by the model, so the model may respond with
    something relevant to the conversation or a prompt to continue speaking.
    """

    audio_end_ms: int
    """
    Millisecond offset of audio written to the input audio buffer at the time the
    timeout was triggered.
    """

    audio_start_ms: int
    """
    Millisecond offset of audio written to the input audio buffer that was after the
    playback time of the last model response.
    """

    event_id: str
    """The unique ID of the server event."""

    item_id: str
    """The ID of the item associated with this segment."""

    type: Literal["input_audio_buffer.timeout_triggered"]
    """The event type, must be `input_audio_buffer.timeout_triggered`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["Project"]


class Project(BaseModel):
    """Represents an individual project."""

    id: str
    """The identifier, which can be referenced in API endpoints"""

    created_at: int
    """The Unix timestamp (in seconds) of when the project was created."""

    object: Literal["organization.project"]
    """The object type, which is always `organization.project`"""

    archived_at: Optional[int] = None
    """The Unix timestamp (in seconds) of when the project was archived or `null`."""

    external_key_id: Optional[str] = None
    """The external key associated with the project."""

    name: Optional[str] = None
    """The name of the project. This appears in reporting."""

    status: Optional[str] = None
    """`active` or `archived`"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RealtimeTruncationRetentionRatio", "TokenLimits"]


class TokenLimits(BaseModel):
    """Optional custom token limits for this truncation strategy.

    If not provided, the model's default token limits will be used.
    """

    post_instructions: Optional[int] = None
    """
    Maximum tokens allowed in the conversation after instructions (which including
    tool definitions). For example, setting this to 5,000 would mean that truncation
    would occur when the conversation exceeds 5,000 tokens after instructions. This
    cannot be higher than the model's context window size minus the maximum output
    tokens.
    """


class RealtimeTruncationRetentionRatio(BaseModel):
    """
    Retain a fraction of the conversation tokens when the conversation exceeds the input token limit. This allows you to amortize truncations across multiple turns, which can help improve cached token usage.
    """

    retention_ratio: float
    """
    Fraction of post-instruction conversation tokens to retain (`0.0` - `1.0`) when
    the conversation exceeds the input token limit. Setting this to `0.8` means that
    messages will be dropped until 80% of the maximum allowed tokens are used. This
    helps reduce the frequency of truncations and improve cache rates.
    """

    type: Literal["retention_ratio"]
    """Use retention ratio truncation."""

    token_limits: Optional[TokenLimits] = None
    """Optional custom token limits for this truncation strategy.

    If not provided, the model's default token limits will be used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .beta_response_input_content import BetaResponseInputContent

__all__ = ["BetaResponseInputMessageContentList"]

BetaResponseInputMessageContentList: TypeAlias = List[BetaResponseInputContent]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .custom_tool import CustomTool

__all__ = ["NamespaceTool", "Tool", "ToolFunction"]


class ToolFunction(BaseModel):
    name: str

    type: Literal["function"]

    allowed_callers: Optional[List[Literal["direct", "programmatic"]]] = None
    """The tool invocation context(s)."""

    defer_loading: Optional[bool] = None
    """Whether this function should be deferred and discovered via tool search."""

    description: Optional[str] = None

    output_schema: Optional[Dict[str, object]] = None
    """
    A JSON Schema describing the JSON value encoded in string outputs for this
    function tool. This does not describe content-array outputs.
    """

    parameters: Optional[object] = None

    strict: Optional[bool] = None
    """Whether to enforce strict parameter validation.

    If omitted, Responses attempts to use strict validation when the schema is
    compatible, and falls back to non-strict validation otherwise.
    """


Tool: TypeAlias = Annotated[Union[ToolFunction, CustomTool], PropertyInfo(discriminator="type")]


class NamespaceTool(BaseModel):
    """Groups function/custom tools under a shared namespace."""

    description: str
    """A description of the namespace shown to the model."""

    name: str
    """The namespace name used in tool calls (for example, `crm`)."""

    tools: List[Tool]
    """The function/custom tools available inside this namespace."""

    type: Literal["namespace"]
    """The type of the tool. Always `namespace`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .api_keys import (
    APIKeys,
    AsyncAPIKeys,
    APIKeysWithRawResponse,
    AsyncAPIKeysWithRawResponse,
    APIKeysWithStreamingResponse,
    AsyncAPIKeysWithStreamingResponse,
)
from .service_accounts import (
    ServiceAccounts,
    AsyncServiceAccounts,
    ServiceAccountsWithRawResponse,
    AsyncServiceAccountsWithRawResponse,
    ServiceAccountsWithStreamingResponse,
    AsyncServiceAccountsWithStreamingResponse,
)

__all__ = [
    "APIKeys",
    "AsyncAPIKeys",
    "APIKeysWithRawResponse",
    "AsyncAPIKeysWithRawResponse",
    "APIKeysWithStreamingResponse",
    "AsyncAPIKeysWithStreamingResponse",
    "ServiceAccounts",
    "AsyncServiceAccounts",
    "ServiceAccountsWithRawResponse",
    "AsyncServiceAccountsWithRawResponse",
    "ServiceAccountsWithStreamingResponse",
    "AsyncServiceAccountsWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["VideoSeconds"]

VideoSeconds: TypeAlias = Literal["4", "8", "12"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from ....._types import SequenceNotStr

__all__ = ["RoleUpdateParams"]


class RoleUpdateParams(TypedDict, total=False):
    project_id: Required[str]

    description: Optional[str]
    """New description for the role."""

    permissions: Optional[SequenceNotStr[str]]
    """Updated set of permissions for the role."""

    role_name: Optional[str]
    """New name for the role."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .reasoning_effort import ReasoningEffort

__all__ = ["Reasoning"]


class Reasoning(BaseModel):
    """**gpt-5 and o-series models only**

    Configuration options for
    [reasoning models](https://platform.openai.com/docs/guides/reasoning).
    """

    context: Optional[Literal["auto", "current_turn", "all_turns"]] = None
    """
    Controls which reasoning items are rendered back to the model on later turns. If
    omitted or set to `auto`, the model determines the context mode. The `gpt-5.6`
    model family defaults to `all_turns`; earlier models default to `current_turn`.

    When returned on a response, this is the effective reasoning context mode used
    for the response.
    """

    effort: Optional[ReasoningEffort] = None
    """Constrains effort on reasoning for reasoning models.

    Currently supported values are `none`, `minimal`, `low`, `medium`, `high`,
    `xhigh`, and `max`. Reducing reasoning effort can result in faster responses and
    fewer tokens used on reasoning in a response. Not all reasoning models support
    every value. See the
    [reasoning guide](https://platform.openai.com/docs/guides/reasoning) for
    model-specific support.
    """

    generate_summary: Optional[Literal["auto", "concise", "detailed"]] = None
    """**Deprecated:** use `summary` instead.

    A summary of the reasoning performed by the model. This can be useful for
    debugging and understanding the model's reasoning process. One of `auto`,
    `concise`, or `detailed`.
    """

    mode: Union[str, Literal["standard", "pro"], None] = None
    """Controls the reasoning execution mode for the request.

    When returned on a response, this is the effective execution mode.
    """

    summary: Optional[Literal["auto", "concise", "detailed"]] = None
    """A summary of the reasoning performed by the model.

    This can be useful for debugging and understanding the model's reasoning
    process. One of `auto`, `concise`, or `detailed`.

    `concise` is supported for `computer-use-preview` models and all reasoning
    models after `gpt-5`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["InlineSkillSource"]


class InlineSkillSource(BaseModel):
    """Inline skill payload"""

    data: str
    """Base64-encoded skill zip bundle."""

    media_type: Literal["application/zip"]
    """The media type of the inline skill payload. Must be `application/zip`."""

    type: Literal["base64"]
    """The type of the inline skill source. Must be `base64`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.types.beta.responses import InputTokenCountResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestInputTokens:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_count(self, client: OpenAI) -> None:
        input_token = client.beta.responses.input_tokens.count()
        assert_matches_type(InputTokenCountResponse, input_token, path=["response"])

    @parametrize
    def test_method_count_with_all_params(self, client: OpenAI) -> None:
        input_token = client.beta.responses.input_tokens.count(
            conversation="string",
            input="string",
            instructions="instructions",
            model="model",
            parallel_tool_calls=True,
            personality="friendly",
            previous_response_id="resp_123",
            reasoning={
                "context": "auto",
                "effort": "none",
                "generate_summary": "auto",
                "mode": "standard",
                "summary": "auto",
            },
            text={
                "format": {"type": "text"},
                "verbosity": "low",
            },
            tool_choice="none",
            tools=[
                {
                    "name": "name",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                    "type": "function",
                    "allowed_callers": ["direct"],
                    "defer_loading": True,
                    "description": "description",
                    "output_schema": {"foo": "bar"},
                }
            ],
            truncation="auto",
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(InputTokenCountResponse, input_token, path=["response"])

    @parametrize
    def test_raw_response_count(self, client: OpenAI) -> None:
        response = client.beta.responses.input_tokens.with_raw_response.count()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        input_token = response.parse()
        assert_matches_type(InputTokenCountResponse, input_token, path=["response"])

    @parametrize
    def test_streaming_response_count(self, client: OpenAI) -> None:
        with client.beta.responses.input_tokens.with_streaming_response.count() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            input_token = response.parse()
            assert_matches_type(InputTokenCountResponse, input_token, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncInputTokens:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_count(self, async_client: AsyncOpenAI) -> None:
        input_token = await async_client.beta.responses.input_tokens.count()
        assert_matches_type(InputTokenCountResponse, input_token, path=["response"])

    @parametrize
    async def test_method_count_with_all_params(self, async_client: AsyncOpenAI) -> None:
        input_token = await async_client.beta.responses.input_tokens.count(
            conversation="string",
            input="string",
            instructions="instructions",
            model="model",
            parallel_tool_calls=True,
            personality="friendly",
            previous_response_id="resp_123",
            reasoning={
                "context": "auto",
                "effort": "none",
                "generate_summary": "auto",
                "mode": "standard",
                "summary": "auto",
            },
            text={
                "format": {"type": "text"},
                "verbosity": "low",
            },
            tool_choice="none",
            tools=[
                {
                    "name": "name",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                    "type": "function",
                    "allowed_callers": ["direct"],
                    "defer_loading": True,
                    "description": "description",
                    "output_schema": {"foo": "bar"},
                }
            ],
            truncation="auto",
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(InputTokenCountResponse, input_token, path=["response"])

    @parametrize
    async def test_raw_response_count(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.input_tokens.with_raw_response.count()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        input_token = response.parse()
        assert_matches_type(InputTokenCountResponse, input_token, path=["response"])

    @parametrize
    async def test_streaming_response_count(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.responses.input_tokens.with_streaming_response.count() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            input_token = await response.parse()
            assert_matches_type(InputTokenCountResponse, input_token, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..... import _legacy_response
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from .....pagination import SyncNextCursorPage, AsyncNextCursorPage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.admin.organization.groups import user_list_params, user_create_params
from .....types.admin.organization.groups.user_create_response import UserCreateResponse
from .....types.admin.organization.groups.user_delete_response import UserDeleteResponse
from .....types.admin.organization.groups.user_retrieve_response import UserRetrieveResponse
from .....types.admin.organization.groups.organization_group_user import OrganizationGroupUser

__all__ = ["Users", "AsyncUsers"]


class Users(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsersWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return UsersWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsersWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return UsersWithStreamingResponse(self)

    def create(
        self,
        group_id: str,
        *,
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserCreateResponse:
        """
        Adds a user to a group.

        Args:
          user_id: Identifier of the user to add to the group.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._post(
            path_template("/organization/groups/{group_id}/users", group_id=group_id),
            body=maybe_transform({"user_id": user_id}, user_create_params.UserCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=UserCreateResponse,
        )

    def retrieve(
        self,
        user_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserRetrieveResponse:
        """
        Retrieves a user in a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            path_template("/organization/groups/{group_id}/users/{user_id}", group_id=group_id, user_id=user_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=UserRetrieveResponse,
        )

    def list(
        self,
        group_id: str,
        *,
        after: str | Omit = omit,
        limit: int | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncNextCursorPage[OrganizationGroupUser]:
        """
        Lists the users assigned to a group.

        Args:
          after: A cursor for use in pagination. Provide the ID of the last user from the
              previous list response to retrieve the next page.

          limit: A limit on the number of users to be returned. Limit can range between 0 and
              1000, and the default is 100.

          order: Specifies the sort order of users in the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._get_api_list(
            path_template("/organization/groups/{group_id}/users", group_id=group_id),
            page=SyncNextCursorPage[OrganizationGroupUser],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                        "order": order,
                    },
                    user_list_params.UserListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=OrganizationGroupUser,
        )

    def delete(
        self,
        user_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserDeleteResponse:
        """
        Removes a user from a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._delete(
            path_template("/organization/groups/{group_id}/users/{user_id}", group_id=group_id, user_id=user_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=UserDeleteResponse,
        )


class AsyncUsers(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsersWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsersWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsersWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncUsersWithStreamingResponse(self)

    async def create(
        self,
        group_id: str,
        *,
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserCreateResponse:
        """
        Adds a user to a group.

        Args:
          user_id: Identifier of the user to add to the group.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return await self._post(
            path_template("/organization/groups/{group_id}/users", group_id=group_id),
            body=await async_maybe_transform({"user_id": user_id}, user_create_params.UserCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=UserCreateResponse,
        )

    async def retrieve(
        self,
        user_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserRetrieveResponse:
        """
        Retrieves a user in a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            path_template("/organization/groups/{group_id}/users/{user_id}", group_id=group_id, user_id=user_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=UserRetrieveResponse,
        )

    def list(
        self,
        group_id: str,
        *,
        after: str | Omit = omit,
        limit: int | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OrganizationGroupUser, AsyncNextCursorPage[OrganizationGroupUser]]:
        """
        Lists the users assigned to a group.

        Args:
          after: A cursor for use in pagination. Provide the ID of the last user from the
              previous list response to retrieve the next page.

          limit: A limit on the number of users to be returned. Limit can range between 0 and
              1000, and the default is 100.

          order: Specifies the sort order of users in the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._get_api_list(
            path_template("/organization/groups/{group_id}/users", group_id=group_id),
            page=AsyncNextCursorPage[OrganizationGroupUser],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                        "order": order,
                    },
                    user_list_params.UserListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=OrganizationGroupUser,
        )

    async def delete(
        self,
        user_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserDeleteResponse:
        """
        Removes a user from a group.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._delete(
            path_template("/organization/groups/{group_id}/users/{user_id}", group_id=group_id, user_id=user_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=UserDeleteResponse,
        )


class UsersWithRawResponse:
    def __init__(self, users: Users) -> None:
        self._users = users

        self.create = _legacy_response.to_raw_response_wrapper(
            users.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            users.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            users.list,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            users.delete,
        )


class AsyncUsersWithRawResponse:
    def __init__(self, users: AsyncUsers) -> None:
        self._users = users

        self.create = _legacy_response.async_to_raw_response_wrapper(
            users.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            users.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            users.list,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            users.delete,
        )


class UsersWithStreamingResponse:
    def __init__(self, users: Users) -> None:
        self._users = users

        self.create = to_streamed_response_wrapper(
            users.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            users.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            users.list,
        )
        self.delete = to_streamed_response_wrapper(
            users.delete,
        )


class AsyncUsersWithStreamingResponse:
    def __init__(self, users: AsyncUsers) -> None:
        self._users = users

        self.create = async_to_streamed_response_wrapper(
            users.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            users.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            users.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            users.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from ..completion_usage import CompletionUsage
from .chat_completion_message import ChatCompletionMessage
from .chat_completion_token_logprob import ChatCompletionTokenLogprob

__all__ = [
    "ChatCompletion",
    "Choice",
    "ChoiceLogprobs",
    "Moderation",
    "ModerationInput",
    "ModerationInputModerationResults",
    "ModerationInputModerationResultsResult",
    "ModerationInputError",
    "ModerationOutput",
    "ModerationOutputModerationResults",
    "ModerationOutputModerationResultsResult",
    "ModerationOutputError",
]


class ChoiceLogprobs(BaseModel):
    """Log probability information for the choice."""

    content: Optional[List[ChatCompletionTokenLogprob]] = None
    """A list of message content tokens with log probability information."""

    refusal: Optional[List[ChatCompletionTokenLogprob]] = None
    """A list of message refusal tokens with log probability information."""


class Choice(BaseModel):
    finish_reason: Literal["stop", "length", "tool_calls", "content_filter", "function_call"]
    """The reason the model stopped generating tokens.

    This will be `stop` if the model hit a natural stop point or a provided stop
    sequence, `length` if the maximum number of tokens specified in the request was
    reached, `content_filter` if content was omitted due to a flag from our content
    filters, `tool_calls` if the model called a tool, or `function_call`
    (deprecated) if the model called a function. Read the
    [Model Spec](https://model-spec.openai.com/2025-12-18.html) for more.
    """

    index: int
    """The index of the choice in the list of choices."""

    logprobs: Optional[ChoiceLogprobs] = None
    """Log probability information for the choice."""

    message: ChatCompletionMessage
    """A chat completion message generated by the model."""


class ModerationInputModerationResultsResult(BaseModel):
    """A moderation result produced for the response input or output."""

    categories: Dict[str, bool]
    """
    A dictionary of moderation categories to booleans, True if the input is flagged
    under this category.
    """

    category_applied_input_types: Dict[str, List[Literal["text", "image"]]]
    """Which modalities of input are reflected by the score for each category."""

    category_scores: Dict[str, float]
    """A dictionary of moderation categories to scores."""

    flagged: bool
    """A boolean indicating whether the content was flagged by any category."""

    model: str
    """The moderation model that produced this result."""

    type: Literal["moderation_result"]
    """
    The object type, which was always `moderation_result` for successful moderation
    results.
    """


class ModerationInputModerationResults(BaseModel):
    """Successful moderation results for the request input or generated output."""

    model: str
    """The moderation model used to generate the results."""

    results: List[ModerationInputModerationResultsResult]
    """A list of moderation results."""

    type: Literal["moderation_results"]
    """The object type, which is always `moderation_results`."""


class ModerationInputError(BaseModel):
    """An error produced while attempting moderation."""

    code: str
    """The error code."""

    message: str
    """The error message."""

    type: Literal["error"]
    """The object type, which is always `error`."""


ModerationInput: TypeAlias = Annotated[
    Union[ModerationInputModerationResults, ModerationInputError], PropertyInfo(discriminator="type")
]


class ModerationOutputModerationResultsResult(BaseModel):
    """A moderation result produced for the response input or output."""

    categories: Dict[str, bool]
    """
    A dictionary of moderation categories to booleans, True if the input is flagged
    under this category.
    """

    category_applied_input_types: Dict[str, List[Literal["text", "image"]]]
    """Which modalities of input are reflected by the score for each category."""

    category_scores: Dict[str, float]
    """A dictionary of moderation categories to scores."""

    flagged: bool
    """A boolean indicating whether the content was flagged by any category."""

    model: str
    """The moderation model that produced this result."""

    type: Literal["moderation_result"]
    """
    The object type, which was always `moderation_result` for successful moderation
    results.
    """


class ModerationOutputModerationResults(BaseModel):
    """Successful moderation results for the request input or generated output."""

    model: str
    """The moderation model used to generate the results."""

    results: List[ModerationOutputModerationResultsResult]
    """A list of moderation results."""

    type: Literal["moderation_results"]
    """The object type, which is always `moderation_results`."""


class ModerationOutputError(BaseModel):
    """An error produced while attempting moderation."""

    code: str
    """The error code."""

    message: str
    """The error message."""

    type: Literal["error"]
    """The object type, which is always `error`."""


ModerationOutput: TypeAlias = Annotated[
    Union[ModerationOutputModerationResults, ModerationOutputError], PropertyInfo(discriminator="type")
]


class Moderation(BaseModel):
    """
    Moderation results for the request input and generated output, if moderated
    completions were requested.
    """

    input: ModerationInput
    """Moderation for the request input."""

    output: ModerationOutput
    """Moderation for the generated output."""


class ChatCompletion(BaseModel):
    """
    Represents a chat completion response returned by model, based on the provided input.
    """

    id: str
    """A unique identifier for the chat completion."""

    choices: List[Choice]
    """A list of chat completion choices.

    Can be more than one if `n` is greater than 1.
    """

    created: int
    """The Unix timestamp (in seconds) of when the chat completion was created."""

    model: str
    """The model used for the chat completion."""

    object: Literal["chat.completion"]
    """The object type, which is always `chat.completion`."""

    moderation: Optional[Moderation] = None
    """
    Moderation results for the request input and generated output, if moderated
    completions were requested.
    """

    service_tier: Optional[Literal["auto", "default", "flex", "scale", "priority", "fast"]] = None
    """Specifies the processing type used for serving the request.

    - If set to 'auto', then the request will be processed with the service tier
      configured in the Project settings. Unless otherwise configured, the Project
      will use 'default'.
    - If set to 'default', then the request will be processed with the standard
      pricing and performance for the selected model.
    - If set to '[flex](https://platform.openai.com/docs/guides/flex-processing)',
      then the request will be processed with the Flex Processing service tier.
    - To opt-in to [Fast mode](/api/docs/guides/fast-mode) at the request level,
      include the `service_tier=fast` or `service_tier=priority` parameter for
      Responses or Chat Completions. The response will show `service_tier=priority`
      regardless of if you specify `service_tier=fast` or `priority` in your
      request.
    - When not set, the default behavior is 'auto'.

    When the `service_tier` parameter is set, the response body will include the
    `service_tier` value based on the processing mode actually used to serve the
    request. This response value may be different from the value set in the
    parameter.
    """

    system_fingerprint: Optional[str] = None
    """This fingerprint represents the backend configuration that the model runs with.

    Can be used in conjunction with the `seed` request parameter to understand when
    backend changes have been made that might impact determinism.
    """

    usage: Optional[CompletionUsage] = None
    """Usage statistics for the completion request."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .threads import (
    Threads,
    AsyncThreads,
    ThreadsWithRawResponse,
    AsyncThreadsWithRawResponse,
    ThreadsWithStreamingResponse,
    AsyncThreadsWithStreamingResponse,
)
from .sessions import (
    Sessions,
    AsyncSessions,
    SessionsWithRawResponse,
    AsyncSessionsWithRawResponse,
    SessionsWithStreamingResponse,
    AsyncSessionsWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ChatKit", "AsyncChatKit"]


class ChatKit(SyncAPIResource):
    @cached_property
    def sessions(self) -> Sessions:
        return Sessions(self._client)

    @cached_property
    def threads(self) -> Threads:
        return Threads(self._client)

    @cached_property
    def with_raw_response(self) -> ChatKitWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return ChatKitWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatKitWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return ChatKitWithStreamingResponse(self)


class AsyncChatKit(AsyncAPIResource):
    @cached_property
    def sessions(self) -> AsyncSessions:
        return AsyncSessions(self._client)

    @cached_property
    def threads(self) -> AsyncThreads:
        return AsyncThreads(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncChatKitWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChatKitWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatKitWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncChatKitWithStreamingResponse(self)


class ChatKitWithRawResponse:
    def __init__(self, chatkit: ChatKit) -> None:
        self._chatkit = chatkit

    @cached_property
    def sessions(self) -> SessionsWithRawResponse:
        return SessionsWithRawResponse(self._chatkit.sessions)

    @cached_property
    def threads(self) -> ThreadsWithRawResponse:
        return ThreadsWithRawResponse(self._chatkit.threads)


class AsyncChatKitWithRawResponse:
    def __init__(self, chatkit: AsyncChatKit) -> None:
        self._chatkit = chatkit

    @cached_property
    def sessions(self) -> AsyncSessionsWithRawResponse:
        return AsyncSessionsWithRawResponse(self._chatkit.sessions)

    @cached_property
    def threads(self) -> AsyncThreadsWithRawResponse:
        return AsyncThreadsWithRawResponse(self._chatkit.threads)


class ChatKitWithStreamingResponse:
    def __init__(self, chatkit: ChatKit) -> None:
        self._chatkit = chatkit

    @cached_property
    def sessions(self) -> SessionsWithStreamingResponse:
        return SessionsWithStreamingResponse(self._chatkit.sessions)

    @cached_property
    def threads(self) -> ThreadsWithStreamingResponse:
        return ThreadsWithStreamingResponse(self._chatkit.threads)


class AsyncChatKitWithStreamingResponse:
    def __init__(self, chatkit: AsyncChatKit) -> None:
        self._chatkit = chatkit

    @cached_property
    def sessions(self) -> AsyncSessionsWithStreamingResponse:
        return AsyncSessionsWithStreamingResponse(self._chatkit.sessions)

    @cached_property
    def threads(self) -> AsyncThreadsWithStreamingResponse:
        return AsyncThreadsWithStreamingResponse(self._chatkit.threads)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from .... import _legacy_response
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ....pagination import SyncCursorPage, AsyncCursorPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.admin.organization import admin_api_key_list_params, admin_api_key_create_params
from ....types.admin.organization.admin_api_key import AdminAPIKey
from ....types.admin.organization.admin_api_key_create_response import AdminAPIKeyCreateResponse
from ....types.admin.organization.admin_api_key_delete_response import AdminAPIKeyDeleteResponse

__all__ = ["AdminAPIKeys", "AsyncAdminAPIKeys"]


class AdminAPIKeys(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AdminAPIKeysWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AdminAPIKeysWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AdminAPIKeysWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AdminAPIKeysWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        expires_in_seconds: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdminAPIKeyCreateResponse:
        """
        Create an organization admin API key

        Args:
          expires_in_seconds: The number of seconds until the API key expires. Omit this field for a key that
              does not expire.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/organization/admin_api_keys",
            body=maybe_transform(
                {
                    "name": name,
                    "expires_in_seconds": expires_in_seconds,
                },
                admin_api_key_create_params.AdminAPIKeyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=AdminAPIKeyCreateResponse,
        )

    def retrieve(
        self,
        key_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdminAPIKey:
        """
        Retrieve a single organization API key

        Args:
          key_id: The ID of the API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return self._get(
            path_template("/organization/admin_api_keys/{key_id}", key_id=key_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=AdminAPIKey,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        limit: int | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[AdminAPIKey]:
        """
        List organization API keys

        Args:
          after: Return keys with IDs that come after this ID in the pagination order.

          limit: Maximum number of keys to return.

          order: Order results by creation time, ascending or descending.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/organization/admin_api_keys",
            page=SyncCursorPage[AdminAPIKey],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                        "order": order,
                    },
                    admin_api_key_list_params.AdminAPIKeyListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=AdminAPIKey,
        )

    def delete(
        self,
        key_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdminAPIKeyDeleteResponse:
        """
        Delete an organization admin API key

        Args:
          key_id: The ID of the API key to be deleted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return self._delete(
            path_template("/organization/admin_api_keys/{key_id}", key_id=key_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=AdminAPIKeyDeleteResponse,
        )


class AsyncAdminAPIKeys(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAdminAPIKeysWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAdminAPIKeysWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAdminAPIKeysWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncAdminAPIKeysWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        expires_in_seconds: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdminAPIKeyCreateResponse:
        """
        Create an organization admin API key

        Args:
          expires_in_seconds: The number of seconds until the API key expires. Omit this field for a key that
              does not expire.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/organization/admin_api_keys",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "expires_in_seconds": expires_in_seconds,
                },
                admin_api_key_create_params.AdminAPIKeyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=AdminAPIKeyCreateResponse,
        )

    async def retrieve(
        self,
        key_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdminAPIKey:
        """
        Retrieve a single organization API key

        Args:
          key_id: The ID of the API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return await self._get(
            path_template("/organization/admin_api_keys/{key_id}", key_id=key_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=AdminAPIKey,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        limit: int | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AdminAPIKey, AsyncCursorPage[AdminAPIKey]]:
        """
        List organization API keys

        Args:
          after: Return keys with IDs that come after this ID in the pagination order.

          limit: Maximum number of keys to return.

          order: Order results by creation time, ascending or descending.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/organization/admin_api_keys",
            page=AsyncCursorPage[AdminAPIKey],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                        "order": order,
                    },
                    admin_api_key_list_params.AdminAPIKeyListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=AdminAPIKey,
        )

    async def delete(
        self,
        key_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdminAPIKeyDeleteResponse:
        """
        Delete an organization admin API key

        Args:
          key_id: The ID of the API key to be deleted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not key_id:
            raise ValueError(f"Expected a non-empty value for `key_id` but received {key_id!r}")
        return await self._delete(
            path_template("/organization/admin_api_keys/{key_id}", key_id=key_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=AdminAPIKeyDeleteResponse,
        )


class AdminAPIKeysWithRawResponse:
    def __init__(self, admin_api_keys: AdminAPIKeys) -> None:
        self._admin_api_keys = admin_api_keys

        self.create = _legacy_response.to_raw_response_wrapper(
            admin_api_keys.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            admin_api_keys.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            admin_api_keys.list,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            admin_api_keys.delete,
        )


class AsyncAdminAPIKeysWithRawResponse:
    def __init__(self, admin_api_keys: AsyncAdminAPIKeys) -> None:
        self._admin_api_keys = admin_api_keys

        self.create = _legacy_response.async_to_raw_response_wrapper(
            admin_api_keys.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            admin_api_keys.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            admin_api_keys.list,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            admin_api_keys.delete,
        )


class AdminAPIKeysWithStreamingResponse:
    def __init__(self, admin_api_keys: AdminAPIKeys) -> None:
        self._admin_api_keys = admin_api_keys

        self.create = to_streamed_response_wrapper(
            admin_api_keys.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            admin_api_keys.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            admin_api_keys.list,
        )
        self.delete = to_streamed_response_wrapper(
            admin_api_keys.delete,
        )


class AsyncAdminAPIKeysWithStreamingResponse:
    def __init__(self, admin_api_keys: AsyncAdminAPIKeys) -> None:
        self._admin_api_keys = admin_api_keys

        self.create = async_to_streamed_response_wrapper(
            admin_api_keys.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            admin_api_keys.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            admin_api_keys.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            admin_api_keys.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChatCompletionContentPartTextParam", "PromptCacheBreakpoint"]


class PromptCacheBreakpoint(TypedDict, total=False):
    """Marks the exact end of a reusable prompt prefix.

    The breakpoint inherits its TTL from the request's `prompt_cache_options.ttl`; the boundary is not rounded to a token block.
    """

    mode: Required[Literal["explicit"]]
    """The breakpoint mode. Always `explicit`."""


class ChatCompletionContentPartTextParam(TypedDict, total=False):
    """
    Learn about [text inputs](https://platform.openai.com/docs/guides/text-generation).
    """

    text: Required[str]
    """The text content."""

    type: Required[Literal["text"]]
    """The type of the content part."""

    prompt_cache_breakpoint: PromptCacheBreakpoint
    """Marks the exact end of a reusable prompt prefix.

    The breakpoint inherits its TTL from the request's `prompt_cache_options.ttl`;
    the boundary is not rounded to a token block.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["ServiceAccountCreateParams"]


class ServiceAccountCreateParams(TypedDict, total=False):
    name: Required[str]
    """The name of the service account being created."""

    create_service_account_only: Optional[bool]
    """Create the service account without default roles or an API key."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["OrganizationUser", "Projects", "ProjectsData", "User"]


class ProjectsData(BaseModel):
    id: Optional[str] = None

    name: Optional[str] = None

    role: Optional[str] = None


class Projects(BaseModel):
    """Projects associated with the user, if included."""

    data: List[ProjectsData]

    object: Literal["list"]


class User(BaseModel):
    """Nested user details."""

    id: str

    object: Literal["user"]

    banned: Optional[bool] = None

    banned_at: Optional[int] = None

    email: Optional[str] = None

    enabled: Optional[bool] = None

    name: Optional[str] = None

    picture: Optional[str] = None


class OrganizationUser(BaseModel):
    """Represents an individual `user` within an organization."""

    id: str
    """The identifier, which can be referenced in API endpoints"""

    added_at: int
    """The Unix timestamp (in seconds) of when the user was added."""

    object: Literal["organization.user"]
    """The object type, which is always `organization.user`"""

    api_key_last_used_at: Optional[int] = None
    """The Unix timestamp (in seconds) of the user's last API key usage."""

    created: Optional[int] = None
    """The Unix timestamp (in seconds) of when the user was created."""

    developer_persona: Optional[str] = None
    """The developer persona metadata for the user."""

    email: Optional[str] = None
    """The email address of the user"""

    is_default: Optional[bool] = None
    """Whether this is the organization's default user."""

    is_scale_tier_authorized_purchaser: Optional[bool] = None
    """Whether the user is an authorized purchaser for Scale Tier."""

    is_scim_managed: Optional[bool] = None
    """Whether the user is managed through SCIM."""

    is_service_account: Optional[bool] = None
    """Whether the user is a service account."""

    name: Optional[str] = None
    """The name of the user"""

    projects: Optional[Projects] = None
    """Projects associated with the user, if included."""

    role: Optional[str] = None
    """`owner` or `reader`"""

    technical_level: Optional[str] = None
    """The technical level metadata for the user."""

    user: Optional[User] = None
    """Nested user details."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .response_function_call_output_item import ResponseFunctionCallOutputItem

__all__ = ["ResponseFunctionCallOutputItemList"]

ResponseFunctionCallOutputItemList: TypeAlias = List[ResponseFunctionCallOutputItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel
from .realtime_response import RealtimeResponse

__all__ = ["ResponseCreatedEvent"]


class ResponseCreatedEvent(BaseModel):
    """Returned when a new Response is created.

    The first event of response creation,
    where the response is in an initial state of `in_progress`.
    """

    event_id: str
    """The unique ID of the server event."""

    response: RealtimeResponse
    """The response resource."""

    type: Literal["response.created"]
    """The event type, must be `response.created`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..... import _legacy_response
from .roles import (
    Roles,
    AsyncRoles,
    RolesWithRawResponse,
    AsyncRolesWithRawResponse,
    RolesWithStreamingResponse,
    AsyncRolesWithStreamingResponse,
)
from .api_keys import (
    APIKeys,
    AsyncAPIKeys,
    APIKeysWithRawResponse,
    AsyncAPIKeysWithRawResponse,
    APIKeysWithStreamingResponse,
    AsyncAPIKeysWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from .rate_limits import (
    RateLimits,
    AsyncRateLimits,
    RateLimitsWithRawResponse,
    AsyncRateLimitsWithRawResponse,
    RateLimitsWithStreamingResponse,
    AsyncRateLimitsWithStreamingResponse,
)
from .spend_limit import (
    SpendLimit,
    AsyncSpendLimit,
    SpendLimitWithRawResponse,
    AsyncSpendLimitWithRawResponse,
    SpendLimitWithStreamingResponse,
    AsyncSpendLimitWithStreamingResponse,
)
from .users.users import (
    Users,
    AsyncUsers,
    UsersWithRawResponse,
    AsyncUsersWithRawResponse,
    UsersWithStreamingResponse,
    AsyncUsersWithStreamingResponse,
)
from .certificates import (
    Certificates,
    AsyncCertificates,
    CertificatesWithRawResponse,
    AsyncCertificatesWithRawResponse,
    CertificatesWithStreamingResponse,
    AsyncCertificatesWithStreamingResponse,
)
from .spend_alerts import (
    SpendAlerts,
    AsyncSpendAlerts,
    SpendAlertsWithRawResponse,
    AsyncSpendAlertsWithRawResponse,
    SpendAlertsWithStreamingResponse,
    AsyncSpendAlertsWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from .groups.groups import (
    Groups,
    AsyncGroups,
    GroupsWithRawResponse,
    AsyncGroupsWithRawResponse,
    GroupsWithStreamingResponse,
    AsyncGroupsWithStreamingResponse,
)
from .....pagination import SyncConversationCursorPage, AsyncConversationCursorPage
from .data_retention import (
    DataRetention,
    AsyncDataRetention,
    DataRetentionWithRawResponse,
    AsyncDataRetentionWithRawResponse,
    DataRetentionWithStreamingResponse,
    AsyncDataRetentionWithStreamingResponse,
)
from ....._base_client import AsyncPaginator, make_request_options
from .model_permissions import (
    ModelPermissions,
    AsyncModelPermissions,
    ModelPermissionsWithRawResponse,
    AsyncModelPermissionsWithRawResponse,
    ModelPermissionsWithStreamingResponse,
    AsyncModelPermissionsWithStreamingResponse,
)
from .hosted_tool_permissions import (
    HostedToolPermissions,
    AsyncHostedToolPermissions,
    HostedToolPermissionsWithRawResponse,
    AsyncHostedToolPermissionsWithRawResponse,
    HostedToolPermissionsWithStreamingResponse,
    AsyncHostedToolPermissionsWithStreamingResponse,
)
from .....types.admin.organization import project_list_params, project_create_params, project_update_params
from .service_accounts.service_accounts import (
    ServiceAccounts,
    AsyncServiceAccounts,
    ServiceAccountsWithRawResponse,
    AsyncServiceAccountsWithRawResponse,
    ServiceAccountsWithStreamingResponse,
    AsyncServiceAccountsWithStreamingResponse,
)
from .....types.admin.organization.project import Project

__all__ = ["Projects", "AsyncProjects"]


class Projects(SyncAPIResource):
    @cached_property
    def users(self) -> Users:
        return Users(self._client)

    @cached_property
    def service_accounts(self) -> ServiceAccounts:
        return ServiceAccounts(self._client)

    @cached_property
    def api_keys(self) -> APIKeys:
        return APIKeys(self._client)

    @cached_property
    def rate_limits(self) -> RateLimits:
        return RateLimits(self._client)

    @cached_property
    def model_permissions(self) -> ModelPermissions:
        return ModelPermissions(self._client)

    @cached_property
    def hosted_tool_permissions(self) -> HostedToolPermissions:
        return HostedToolPermissions(self._client)

    @cached_property
    def groups(self) -> Groups:
        return Groups(self._client)

    @cached_property
    def roles(self) -> Roles:
        return Roles(self._client)

    @cached_property
    def data_retention(self) -> DataRetention:
        return DataRetention(self._client)

    @cached_property
    def spend_limit(self) -> SpendLimit:
        return SpendLimit(self._client)

    @cached_property
    def spend_alerts(self) -> SpendAlerts:
        return SpendAlerts(self._client)

    @cached_property
    def certificates(self) -> Certificates:
        return Certificates(self._client)

    @cached_property
    def with_raw_response(self) -> ProjectsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return ProjectsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProjectsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return ProjectsWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        external_key_id: Optional[str] | Omit = omit,
        geography: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Project:
        """Create a new project in the organization.

        Projects can be created and archived,
        but cannot be deleted.

        Args:
          name: The friendly name of the project, this name appears in reports.

          external_key_id: External key ID to associate with the project.

          geography: Create the project with the specified data residency region. Your organization
              must have access to Data residency functionality in order to use. See
              [data residency controls](https://platform.openai.com/docs/guides/your-data#data-residency-controls)
              to review the functionality and limitations of setting this field.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/organization/projects",
            body=maybe_transform(
                {
                    "name": name,
                    "external_key_id": external_key_id,
                    "geography": geography,
                },
                project_create_params.ProjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=Project,
        )

    def retrieve(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Project:
        """
        Retrieves a project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._get(
            path_template("/organization/projects/{project_id}", project_id=project_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=Project,
        )

    def update(
        self,
        project_id: str,
        *,
        external_key_id: Optional[str] | Omit = omit,
        geography: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Project:
        """
        Modifies a project in the organization.

        Args:
          external_key_id: External key ID to associate with the project.

          geography: Geography for the project.

          name: The updated name of the project, this name appears in reports.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._post(
            path_template("/organization/projects/{project_id}", project_id=project_id),
            body=maybe_transform(
                {
                    "external_key_id": external_key_id,
                    "geography": geography,
                    "name": name,
                },
                project_update_params.ProjectUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=Project,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        include_archived: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncConversationCursorPage[Project]:
        """Returns a list of projects.

        Args:
          after: A cursor for use in pagination.

        `after` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              ending with obj_foo, your subsequent call can include after=obj_foo in order to
              fetch the next page of the list.

          include_archived: If `true` returns all projects including those that have been `archived`.
              Archived projects are not included by default.

          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              100, and the default is 20.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/organization/projects",
            page=SyncConversationCursorPage[Project],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "include_archived": include_archived,
                        "limit": limit,
                    },
                    project_list_params.ProjectListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=Project,
        )

    def archive(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Project:
        """Archives a project in the organization.

        Archived projects cannot be used or
        updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._post(
            path_template("/organization/projects/{project_id}/archive", project_id=project_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=Project,
        )


class AsyncProjects(AsyncAPIResource):
    @cached_property
    def users(self) -> AsyncUsers:
        return AsyncUsers(self._client)

    @cached_property
    def service_accounts(self) -> AsyncServiceAccounts:
        return AsyncServiceAccounts(self._client)

    @cached_property
    def api_keys(self) -> AsyncAPIKeys:
        return AsyncAPIKeys(self._client)

    @cached_property
    def rate_limits(self) -> AsyncRateLimits:
        return AsyncRateLimits(self._client)

    @cached_property
    def model_permissions(self) -> AsyncModelPermissions:
        return AsyncModelPermissions(self._client)

    @cached_property
    def hosted_tool_permissions(self) -> AsyncHostedToolPermissions:
        return AsyncHostedToolPermissions(self._client)

    @cached_property
    def groups(self) -> AsyncGroups:
        return AsyncGroups(self._client)

    @cached_property
    def roles(self) -> AsyncRoles:
        return AsyncRoles(self._client)

    @cached_property
    def data_retention(self) -> AsyncDataRetention:
        return AsyncDataRetention(self._client)

    @cached_property
    def spend_limit(self) -> AsyncSpendLimit:
        return AsyncSpendLimit(self._client)

    @cached_property
    def spend_alerts(self) -> AsyncSpendAlerts:
        return AsyncSpendAlerts(self._client)

    @cached_property
    def certificates(self) -> AsyncCertificates:
        return AsyncCertificates(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncProjectsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProjectsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProjectsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncProjectsWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        external_key_id: Optional[str] | Omit = omit,
        geography: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Project:
        """Create a new project in the organization.

        Projects can be created and archived,
        but cannot be deleted.

        Args:
          name: The friendly name of the project, this name appears in reports.

          external_key_id: External key ID to associate with the project.

          geography: Create the project with the specified data residency region. Your organization
              must have access to Data residency functionality in order to use. See
              [data residency controls](https://platform.openai.com/docs/guides/your-data#data-residency-controls)
              to review the functionality and limitations of setting this field.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/organization/projects",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "external_key_id": external_key_id,
                    "geography": geography,
                },
                project_create_params.ProjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=Project,
        )

    async def retrieve(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Project:
        """
        Retrieves a project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return await self._get(
            path_template("/organization/projects/{project_id}", project_id=project_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=Project,
        )

    async def update(
        self,
        project_id: str,
        *,
        external_key_id: Optional[str] | Omit = omit,
        geography: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Project:
        """
        Modifies a project in the organization.

        Args:
          external_key_id: External key ID to associate with the project.

          geography: Geography for the project.

          name: The updated name of the project, this name appears in reports.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return await self._post(
            path_template("/organization/projects/{project_id}", project_id=project_id),
            body=await async_maybe_transform(
                {
                    "external_key_id": external_key_id,
                    "geography": geography,
                    "name": name,
                },
                project_update_params.ProjectUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=Project,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        include_archived: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Project, AsyncConversationCursorPage[Project]]:
        """Returns a list of projects.

        Args:
          after: A cursor for use in pagination.

        `after` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              ending with obj_foo, your subsequent call can include after=obj_foo in order to
              fetch the next page of the list.

          include_archived: If `true` returns all projects including those that have been `archived`.
              Archived projects are not included by default.

          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              100, and the default is 20.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/organization/projects",
            page=AsyncConversationCursorPage[Project],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "include_archived": include_archived,
                        "limit": limit,
                    },
                    project_list_params.ProjectListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=Project,
        )

    async def archive(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Project:
        """Archives a project in the organization.

        Archived projects cannot be used or
        updated.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return await self._post(
            path_template("/organization/projects/{project_id}/archive", project_id=project_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=Project,
        )


class ProjectsWithRawResponse:
    def __init__(self, projects: Projects) -> None:
        self._projects = projects

        self.create = _legacy_response.to_raw_response_wrapper(
            projects.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            projects.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            projects.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            projects.list,
        )
        self.archive = _legacy_response.to_raw_response_wrapper(
            projects.archive,
        )

    @cached_property
    def users(self) -> UsersWithRawResponse:
        return UsersWithRawResponse(self._projects.users)

    @cached_property
    def service_accounts(self) -> ServiceAccountsWithRawResponse:
        return ServiceAccountsWithRawResponse(self._projects.service_accounts)

    @cached_property
    def api_keys(self) -> APIKeysWithRawResponse:
        return APIKeysWithRawResponse(self._projects.api_keys)

    @cached_property
    def rate_limits(self) -> RateLimitsWithRawResponse:
        return RateLimitsWithRawResponse(self._projects.rate_limits)

    @cached_property
    def model_permissions(self) -> ModelPermissionsWithRawResponse:
        return ModelPermissionsWithRawResponse(self._projects.model_permissions)

    @cached_property
    def hosted_tool_permissions(self) -> HostedToolPermissionsWithRawResponse:
        return HostedToolPermissionsWithRawResponse(self._projects.hosted_tool_permissions)

    @cached_property
    def groups(self) -> GroupsWithRawResponse:
        return GroupsWithRawResponse(self._projects.groups)

    @cached_property
    def roles(self) -> RolesWithRawResponse:
        return RolesWithRawResponse(self._projects.roles)

    @cached_property
    def data_retention(self) -> DataRetentionWithRawResponse:
        return DataRetentionWithRawResponse(self._projects.data_retention)

    @cached_property
    def spend_limit(self) -> SpendLimitWithRawResponse:
        return SpendLimitWithRawResponse(self._projects.spend_limit)

    @cached_property
    def spend_alerts(self) -> SpendAlertsWithRawResponse:
        return SpendAlertsWithRawResponse(self._projects.spend_alerts)

    @cached_property
    def certificates(self) -> CertificatesWithRawResponse:
        return CertificatesWithRawResponse(self._projects.certificates)


class AsyncProjectsWithRawResponse:
    def __init__(self, projects: AsyncProjects) -> None:
        self._projects = projects

        self.create = _legacy_response.async_to_raw_response_wrapper(
            projects.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            projects.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            projects.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            projects.list,
        )
        self.archive = _legacy_response.async_to_raw_response_wrapper(
            projects.archive,
        )

    @cached_property
    def users(self) -> AsyncUsersWithRawResponse:
        return AsyncUsersWithRawResponse(self._projects.users)

    @cached_property
    def service_accounts(self) -> AsyncServiceAccountsWithRawResponse:
        return AsyncServiceAccountsWithRawResponse(self._projects.service_accounts)

    @cached_property
    def api_keys(self) -> AsyncAPIKeysWithRawResponse:
        return AsyncAPIKeysWithRawResponse(self._projects.api_keys)

    @cached_property
    def rate_limits(self) -> AsyncRateLimitsWithRawResponse:
        return AsyncRateLimitsWithRawResponse(self._projects.rate_limits)

    @cached_property
    def model_permissions(self) -> AsyncModelPermissionsWithRawResponse:
        return AsyncModelPermissionsWithRawResponse(self._projects.model_permissions)

    @cached_property
    def hosted_tool_permissions(self) -> AsyncHostedToolPermissionsWithRawResponse:
        return AsyncHostedToolPermissionsWithRawResponse(self._projects.hosted_tool_permissions)

    @cached_property
    def groups(self) -> AsyncGroupsWithRawResponse:
        return AsyncGroupsWithRawResponse(self._projects.groups)

    @cached_property
    def roles(self) -> AsyncRolesWithRawResponse:
        return AsyncRolesWithRawResponse(self._projects.roles)

    @cached_property
    def data_retention(self) -> AsyncDataRetentionWithRawResponse:
        return AsyncDataRetentionWithRawResponse(self._projects.data_retention)

    @cached_property
    def spend_limit(self) -> AsyncSpendLimitWithRawResponse:
        return AsyncSpendLimitWithRawResponse(self._projects.spend_limit)

    @cached_property
    def spend_alerts(self) -> AsyncSpendAlertsWithRawResponse:
        return AsyncSpendAlertsWithRawResponse(self._projects.spend_alerts)

    @cached_property
    def certificates(self) -> AsyncCertificatesWithRawResponse:
        return AsyncCertificatesWithRawResponse(self._projects.certificates)


class ProjectsWithStreamingResponse:
    def __init__(self, projects: Projects) -> None:
        self._projects = projects

        self.create = to_streamed_response_wrapper(
            projects.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            projects.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            projects.update,
        )
        self.list = to_streamed_response_wrapper(
            projects.list,
        )
        self.archive = to_streamed_response_wrapper(
            projects.archive,
        )

    @cached_property
    def users(self) -> UsersWithStreamingResponse:
        return UsersWithStreamingResponse(self._projects.users)

    @cached_property
    def service_accounts(self) -> ServiceAccountsWithStreamingResponse:
        return ServiceAccountsWithStreamingResponse(self._projects.service_accounts)

    @cached_property
    def api_keys(self) -> APIKeysWithStreamingResponse:
        return APIKeysWithStreamingResponse(self._projects.api_keys)

    @cached_property
    def rate_limits(self) -> RateLimitsWithStreamingResponse:
        return RateLimitsWithStreamingResponse(self._projects.rate_limits)

    @cached_property
    def model_permissions(self) -> ModelPermissionsWithStreamingResponse:
        return ModelPermissionsWithStreamingResponse(self._projects.model_permissions)

    @cached_property
    def hosted_tool_permissions(self) -> HostedToolPermissionsWithStreamingResponse:
        return HostedToolPermissionsWithStreamingResponse(self._projects.hosted_tool_permissions)

    @cached_property
    def groups(self) -> GroupsWithStreamingResponse:
        return GroupsWithStreamingResponse(self._projects.groups)

    @cached_property
    def roles(self) -> RolesWithStreamingResponse:
        return RolesWithStreamingResponse(self._projects.roles)

    @cached_property
    def data_retention(self) -> DataRetentionWithStreamingResponse:
        return DataRetentionWithStreamingResponse(self._projects.data_retention)

    @cached_property
    def spend_limit(self) -> SpendLimitWithStreamingResponse:
        return SpendLimitWithStreamingResponse(self._projects.spend_limit)

    @cached_property
    def spend_alerts(self) -> SpendAlertsWithStreamingResponse:
        return SpendAlertsWithStreamingResponse(self._projects.spend_alerts)

    @cached_property
    def certificates(self) -> CertificatesWithStreamingResponse:
        return CertificatesWithStreamingResponse(self._projects.certificates)


class AsyncProjectsWithStreamingResponse:
    def __init__(self, projects: AsyncProjects) -> None:
        self._projects = projects

        self.create = async_to_streamed_response_wrapper(
            projects.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            projects.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            projects.update,
        )
        self.list = async_to_streamed_response_wrapper(
            projects.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            projects.archive,
        )

    @cached_property
    def users(self) -> AsyncUsersWithStreamingResponse:
        return AsyncUsersWithStreamingResponse(self._projects.users)

    @cached_property
    def service_accounts(self) -> AsyncServiceAccountsWithStreamingResponse:
        return AsyncServiceAccountsWithStreamingResponse(self._projects.service_accounts)

    @cached_property
    def api_keys(self) -> AsyncAPIKeysWithStreamingResponse:
        return AsyncAPIKeysWithStreamingResponse(self._projects.api_keys)

    @cached_property
    def rate_limits(self) -> AsyncRateLimitsWithStreamingResponse:
        return AsyncRateLimitsWithStreamingResponse(self._projects.rate_limits)

    @cached_property
    def model_permissions(self) -> AsyncModelPermissionsWithStreamingResponse:
        return AsyncModelPermissionsWithStreamingResponse(self._projects.model_permissions)

    @cached_property
    def hosted_tool_permissions(self) -> AsyncHostedToolPermissionsWithStreamingResponse:
        return AsyncHostedToolPermissionsWithStreamingResponse(self._projects.hosted_tool_permissions)

    @cached_property
    def groups(self) -> AsyncGroupsWithStreamingResponse:
        return AsyncGroupsWithStreamingResponse(self._projects.groups)

    @cached_property
    def roles(self) -> AsyncRolesWithStreamingResponse:
        return AsyncRolesWithStreamingResponse(self._projects.roles)

    @cached_property
    def data_retention(self) -> AsyncDataRetentionWithStreamingResponse:
        return AsyncDataRetentionWithStreamingResponse(self._projects.data_retention)

    @cached_property
    def spend_limit(self) -> AsyncSpendLimitWithStreamingResponse:
        return AsyncSpendLimitWithStreamingResponse(self._projects.spend_limit)

    @cached_property
    def spend_alerts(self) -> AsyncSpendAlertsWithStreamingResponse:
        return AsyncSpendAlertsWithStreamingResponse(self._projects.spend_alerts)

    @cached_property
    def certificates(self) -> AsyncCertificatesWithStreamingResponse:
        return AsyncCertificatesWithStreamingResponse(self._projects.certificates)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr
from .beta_container_network_policy_domain_secret_param import BetaContainerNetworkPolicyDomainSecretParam

__all__ = ["BetaContainerNetworkPolicyAllowlistParam"]


class BetaContainerNetworkPolicyAllowlistParam(TypedDict, total=False):
    allowed_domains: Required[SequenceNotStr[str]]
    """A list of allowed domains when type is `allowlist`."""

    type: Required[Literal["allowlist"]]
    """Allow outbound network access only to specified domains. Always `allowlist`."""

    domain_secrets: Iterable[BetaContainerNetworkPolicyDomainSecretParam]
    """Optional domain-scoped secrets for allowlisted domains."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from .beta_response_input_file_content import BetaResponseInputFileContent
from .beta_response_input_text_content import BetaResponseInputTextContent
from .beta_response_input_image_content import BetaResponseInputImageContent

__all__ = ["BetaResponseFunctionCallOutputItem"]

BetaResponseFunctionCallOutputItem: TypeAlias = Annotated[
    Union[BetaResponseInputTextContent, BetaResponseInputImageContent, BetaResponseInputFileContent],
    PropertyInfo(discriminator="type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["ProjectCreateParams"]


class ProjectCreateParams(TypedDict, total=False):
    name: Required[str]
    """The friendly name of the project, this name appears in reports."""

    external_key_id: Optional[str]
    """External key ID to associate with the project."""

    geography: Optional[str]
    """Create the project with the specified data residency region.

    Your organization must have access to Data residency functionality in order to
    use. See
    [data residency controls](https://platform.openai.com/docs/guides/your-data#data-residency-controls)
    to review the functionality and limitations of setting this field.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..role import Role
from ....._models import BaseModel
from ..organization_user import OrganizationUser

__all__ = ["RoleCreateResponse"]


class RoleCreateResponse(BaseModel):
    """Role assignment linking a user to a role."""

    object: Literal["user.role"]
    """Always `user.role`."""

    role: Role
    """Details about a role that can be assigned through the public Roles API."""

    user: OrganizationUser
    """Represents an individual `user` within an organization."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["Group"]


class Group(BaseModel):
    """Details about an organization group."""

    id: str
    """Identifier for the group."""

    created_at: int
    """Unix timestamp (in seconds) when the group was created."""

    group_type: Literal["group", "tenant_group"]
    """The type of the group."""

    is_scim_managed: bool
    """
    Whether the group is managed through SCIM and controlled by your identity
    provider.
    """

    name: str
    """Display name of the group."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["RoleListParams"]


class RoleListParams(TypedDict, total=False):
    after: str
    """Cursor for pagination.

    Provide the value from the previous response's `next` field to continue listing
    organization roles.
    """

    limit: int
    """A limit on the number of organization role assignments to return."""

    order: Literal["asc", "desc"]
    """Sort order for the returned organization roles."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .beta_response_format_text_config_param import BetaResponseFormatTextConfigParam

__all__ = ["BetaResponseTextConfigParam"]


class BetaResponseTextConfigParam(TypedDict, total=False):
    """Configuration options for a text response from the model.

    Can be plain
    text or structured JSON data. Learn more:
    - [Text inputs and outputs](https://platform.openai.com/docs/guides/text)
    - [Structured Outputs](https://platform.openai.com/docs/guides/structured-outputs)
    """

    format: BetaResponseFormatTextConfigParam
    """An object specifying the format that the model must output.

    Configuring `{ "type": "json_schema" }` enables Structured Outputs, which
    ensures the model will match your supplied JSON schema. Learn more in the
    [Structured Outputs guide](https://platform.openai.com/docs/guides/structured-outputs).

    The default format is `{ "type": "text" }` with no additional options.

    **Not recommended for gpt-4o and newer models:**

    Setting to `{ "type": "json_object" }` enables the older JSON mode, which
    ensures the message the model generates is valid JSON. Using `json_schema` is
    preferred for models that support it.
    """

    verbosity: Optional[Literal["low", "medium", "high"]]
    """Constrains the verbosity of the model's response.

    Lower values will result in more concise responses, while higher values will
    result in more verbose responses. Currently supported values are `low`,
    `medium`, and `high`. The default is `medium`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BetaResponseErrorEvent", "Agent"]


class Agent(BaseModel):
    """The agent that owns this multi-agent streaming event."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class BetaResponseErrorEvent(BaseModel):
    """Emitted when an error occurs."""

    code: Optional[str] = None
    """The error code."""

    message: str
    """The error message."""

    param: Optional[str] = None
    """The error parameter."""

    sequence_number: int
    """The sequence number of this event."""

    type: Literal["error"]
    """The type of the event. Always `error`."""

    agent: Optional[Agent] = None
    """The agent that owns this multi-agent streaming event."""
